package com.usky.system.service;

import com.usky.system.domain.SysMenu;
import com.usky.system.domain.SysMobileMenu;
import com.usky.common.mybatis.core.CrudService;
import com.usky.system.domain.SysMobileTenantMenu;
import com.usky.system.domain.SysTenantMenu;
import com.usky.system.service.vo.RouterVo;
import com.usky.system.service.vo.TreeMobileSelect;

import java.util.List;

/**
 * <p>
 * 移动端_菜单权限表 服务类
 * </p>
 *
 * @author JCB
 * @since 2022-08-11
 */
public interface SysMobileMenuService extends CrudService<SysMobileMenu> {

    /**
     * 移动端-首页路由
     * @return
     */
    List<SysMobileMenu> moveRoute();

    /**
     * 根据用户Id获取菜单树信息
     *
     * @param userId 用户Id
     * @return 菜单列表
     */
    List<SysMobileMenu> selectMenuTreeByUserId(Long userId);

    /**
     * 根据租户Id获取菜单树信息
     *
     * @param tenantId 租户Id
     * @return 菜单列表
     */
    List<SysMobileMenu> selectMenuTreeByUserIdOne(Integer tenantId);


    List<SysMobileTenantMenu> getMobileTenantMenuList(Integer tenantId);

    void updateMobileTenantMenu(SysMobileTenantMenu mobileTenantMenu);

    /**
     * 根据角色ID查询菜单树信息
     *
     * @param roleId 角色ID
     * @return 选中菜单列表
     */
    List<Integer> selectMenuListByRoleId(Long roleId);


    /**
     * 构建前端所需要树结构
     *
     * @param menus 菜单列表
     * @return 树结构列表
     */
    public List<SysMobileMenu> buildMenuTree(List<SysMobileMenu> menus);

    /**
     * 构建前端路由所需要的菜单
     *
     * @param menus 菜单列表
     * @return 路由列表
     */
    List<RouterVo> buildMenus(List<SysMobileMenu> menus);

    /**
     * 根据租户Id获取展示菜单列表
     * @return
     */
    List<SysMobileMenu> selectMenuListTwo();

    /**
     * 构建前端所需要下拉树结构
     *
     * @param menus 菜单列表
     * @return 下拉树结构列表
     */
    public List<TreeMobileSelect> buildMenuTreeSelect(List<SysMobileMenu> menus);

    /**
     * 根据用户查询系统菜单列表
     *
     * @param menu 菜单信息
     * @param userId 用户ID
     * @return 菜单列表
     */
    public List<SysMobileMenu> selectMobileMenuList(SysMobileMenu menu, Long userId);

    /**
     * 根据菜单ID查询信息
     *
     * @param menuId 菜单ID
     * @return 菜单信息
     */
    public SysMobileMenu selectMenuById(Long menuId);

    /**
     * 校验菜单名称是否唯一
     *
     * @param menu 菜单信息
     * @return 结果
     */
    public String checkMenuNameUnique(SysMobileMenu menu);

    /**
     * 新增保存菜单信息
     *
     * @param menu 菜单信息
     * @return 结果
     */
    public int insertMobileMenu(SysMobileMenu menu);

    /**
     * 修改保存菜单信息
     *
     * @param menu 菜单信息
     * @return 结果
     */
    public int updateMenu(SysMobileMenu menu);

    /**
     * 删除菜单管理信息
     *
     * @param menuId 菜单ID
     * @return 结果
     */
    public int deleteMenuById(Long menuId);

    /**
     * 是否存在菜单子节点
     *
     * @param menuId 菜单ID
     * @return 结果 true 存在 false 不存在
     */
    public boolean hasChildByMenuId(Long menuId);

    /**
     * 查询菜单是否存在角色
     *
     * @param menuId 菜单ID
     * @return 结果 true 存在 false 不存在
     */
    public boolean checkMenuExistRole(Long menuId);
}
